package com.hyperion.essbase.cdf.export;

import java.sql.*;
import java.util.*;
import java.text.*;
import java.io.*;


/**
 * Title:        ExportTo
 * Description:  Exports data from essbase to a text file, a dsn (using a JBDC-ODBC bridge) or a rdb (using JDBC)
 * Copyright:
 * Company:      Hyperion Solutions
 * @author:      Toufic Wakim
 * @version 2.0
 * Disclaimer: use provided functions at your own risk. Hyperion Solutions Corp. is not liable for any
  damage, direct or indirect, associated with the use of the supplied functions.
 */

public class ExportTo {

  static Map dsnMap = new HashMap();
  static Map fileMap = new HashMap();
  static Map sortMap=new HashMap();
  static Connection sqlcon=null;
  static Statement sqlstmt = null;
  static NumberFormat numberFormatter;
  static Locale currentLocale=new Locale("en","US");
  static NumberFormat nf = NumberFormat.getNumberInstance(currentLocale);
  static DecimalFormat df = (DecimalFormat)nf;


public static void main(String args[]) {
	String[] strArgs = {"New york"};
	double[] dblArgs = {1,2,3,4,5};
	int[] decimalPoints = {1,2,3,4};
	String [] replaceMiss = {"M","M","M","M","M"};
	ExportToFile("c:/flatdec.txt","tab","quotes",strArgs, dblArgs,decimalPoints,replaceMiss);

}
  public static void ExportToTarget(String targetType, String targetName, String delimiter,
                                  String tableName, String[] strArgs, double[] dblArgs) {

          if (targetType.equalsIgnoreCase("file")) {

              ExportToFile(targetName,delimiter,strArgs,dblArgs);

          } else if (targetType.equalsIgnoreCase("dsn")) {

              ExportToDSN(targetName,tableName,strArgs,dblArgs);

          } else if (targetType.equalsIgnoreCase("JDBC")) {

              ExportToJDBC(targetName,tableName,strArgs,dblArgs);
          }

  }

  public static void ExportToTarget(String targetType, String targetName, String delimiter, String textQualifier,
                                  String tableName, String[] strArgs, double[] dblArgs,int[] decimalPoints,
                                  String[] replaceMiss) {

          if (targetType.equalsIgnoreCase("file")) {

              ExportToFile(targetName,delimiter,textQualifier,strArgs,dblArgs,decimalPoints,replaceMiss);

          } else if (targetType.equalsIgnoreCase("dsn")) {

              ExportToDSN(targetName,tableName,strArgs,dblArgs,decimalPoints,replaceMiss);

          } else if (targetType.equalsIgnoreCase("JDBC")) {

              ExportToJDBC(targetName,tableName,strArgs,dblArgs,decimalPoints,replaceMiss);
          }

  }

  private static void ExportToFile(String fileName, String delimiter, String[] strArgs, double[] dblArgs) {

        openFile(fileName);
        insertToFile(fileName, delimiter,strArgs,dblArgs);
  }

   private static void ExportToFile(String fileName, String delimiter, String textQualifier,String[] strArgs, double[] dblArgs, int[] decimalPoints,String[] replaceMiss) {

        openFile(fileName);
        insertToFile(fileName, delimiter,textQualifier,strArgs,dblArgs,decimalPoints,replaceMiss);
  }

  static void ExportLineToFile(String fileName, String lineToExport) {

		  openFile(fileName);
		  writeToFile(fileName, lineToExport + "\n");
	}
  private static void ExportToDSN(String targetName, String tableName, String[] strArgs, double[] dblArgs) {

        establishConnection("dsn",targetName);
        insertIntoTable(targetName, tableName,strArgs,dblArgs);
}

private static void ExportToDSN(String targetName, String tableName, String[] strArgs, double[] dblArgs,int[] decimalPoints,String[] replaceMiss) {

        establishConnection("dsn",targetName);
        insertIntoTable(targetName, tableName,strArgs,dblArgs,decimalPoints,replaceMiss);
}
 private static void ExportToJDBC(String targetName, String tableName, String[] strArgs, double[] dblArgs) {

        establishConnection("jdbc",targetName);
        insertIntoTable(targetName, tableName,strArgs,dblArgs);
}

private static void ExportToJDBC(String targetName, String tableName, String[] strArgs, double[] dblArgs,int[] decimalPoints,String[] replaceMiss) {

        establishConnection("jdbc",targetName);
        insertIntoTable(targetName, tableName,strArgs,dblArgs,decimalPoints,replaceMiss);
}

  private static void openFile(String fileName) {

         if (!fileMap.containsKey(fileName)) {

               // make a connection to the selected data source.
               try {
                    fileMap.put(fileName,new FileWriter(new File(fileName)));
                    System.out.println(fileName + ": file opened.");
               } catch (Throwable t) {
                    System.out.println(fileName + ": Can't open file " + t.getMessage());
               }
         }
  }

  private static void insertToFile(String fileName, String delimiter, String textQualifier,
  				String[] strArgs, double[] dblArgs, int[] decimalPoints, String[] replaceMiss) {

        StringBuffer insertString=new StringBuffer();
        String numToFormat=null;
        boolean hasString=true;
        boolean hasDouble=true;

        if (strArgs.length==1 && strArgs[0].trim().length()==0) {
          hasString=false;
        }

        if (dblArgs.length==1 && dblArgs[0]==0 ) {
          hasDouble=false;
        }

        try {

              if (hasString ) {
                  for (int i = 0; i < strArgs.length; i++) {
                      if ((i==strArgs.length -1) && !hasDouble) {
                          insertString.append((textQualifier.equalsIgnoreCase("quotes")?"\"":textQualifier) + strArgs[i] + (textQualifier.equalsIgnoreCase("quotes")?"\"":textQualifier) + "\n");
                      } else {
                          insertString.append((textQualifier.equalsIgnoreCase("quotes")?"\"":textQualifier) + strArgs[i] + (textQualifier.equalsIgnoreCase("quotes")?"\"":textQualifier) + (delimiter.equalsIgnoreCase("tab")?"\t":delimiter));
                      }

                  }
              }

              if(hasDouble) {
              	  String replaceMissing = null;
              	  int precision = 0; 
                  for (int i = 0; i < dblArgs.length; i++) {
                  	 if (replaceMiss.length < (i+1))
					 	replaceMissing = replaceMiss[replaceMiss.length -1];
					 else
						replaceMissing = replaceMiss[i];
					
					 if (decimalPoints.length < (i+1))
						precision = decimalPoints[decimalPoints.length -1];
					 else
						precision = decimalPoints[i] ;
					 
                     if (com.hyperion.essbase.calculator.Calculator.isMissing(dblArgs[i]))
                        try {
                           	numToFormat= new RoundDouble().getRoundAsString(Double.parseDouble(replaceMissing),precision) ;
                        } catch (Exception x) {
                           	if (replaceMissing.length() == 0)
								numToFormat = "#Miss";	
                           	else
                               	numToFormat = replaceMissing;
                        }
                     else
                        numToFormat=new RoundDouble().getRoundAsString(dblArgs[i],precision) ;

                     if (i==dblArgs.length -1) {
                       insertString.append(numToFormat + "\n");
                     } else {
                       insertString.append(numToFormat + (delimiter.equalsIgnoreCase("tab")?"\t":delimiter));
                     }
                  }
              }

          if (hasString || hasDouble)
              writeToFile(fileName,insertString.toString() ) ;

        } catch(Exception e) {
             System.out.println(e.getMessage());

        } finally {

           insertString=null;
           numToFormat=null;
        }


  }

  private static void writeToFile(String fileName, String output) {

        FileWriter thisOut=null;
        try {

          thisOut= (FileWriter) fileMap.get(fileName);
          thisOut.write(output ) ;

        } catch(Exception e) {
             System.out.println(e.getMessage());

        } finally {
            thisOut=null;
        }


  }
  private static void insertToFile(String fileName, String delimiter, String[] strArgs, double[] dblArgs) {

        StringBuffer insertString=new StringBuffer();
        String delimit=delimiter;
        boolean hasString=true;
        boolean hasDouble=true;

        if (strArgs.length==1 && strArgs[0].trim().length()==0) {
          hasString=false;
        }

        if (dblArgs.length==1 && dblArgs[0]==0 ) {
          hasDouble=false;
        }

        df.applyPattern("##0.00");

        if (delimiter.equalsIgnoreCase("tab"))
            delimit="\t";

        try {

              if (hasString ) {
                  for (int i = 0; i < strArgs.length; i++) {
                      if ((i==strArgs.length -1) && !hasDouble) {
                          insertString.append("\"" + strArgs[i] + "\"" + "\n");
                      } else {
                          insertString.append("\"" + strArgs[i] + "\"" + delimit);
                      }

                  }
              }

              if (hasDouble) {
                  for (int i = 0; i < dblArgs.length; i++) {
                     if (i==dblArgs.length -1) {
                       insertString.append(df.format(dblArgs[i] ) + "\n");
                     } else {
                       insertString.append(df.format(dblArgs[i] )+ delimit);
                     }
                  }
              }

              if (hasString || hasDouble)
                  writeToFile(fileName,insertString.toString() ) ;

        } catch(Exception e) {
             System.out.println(e.getMessage());

        }


  }
  private static void establishConnection(String targetType, String targetName) {

         String mySQLurl =null;
         String mySQLDriver=null;
         String myUser="";
         String myPassword="";

         if (targetType.equalsIgnoreCase("dsn")) {
              mySQLurl = "JDBC:ODBC:" + targetName;
              mySQLDriver="sun.jdbc.odbc.JdbcOdbcDriver";
         } else {
              ResourceBundle rbRDBMS= ResourceBundle.getBundle("rdb");
              mySQLurl=rbRDBMS.getString(targetName+ ".url");
              mySQLDriver=rbRDBMS.getString(targetName+ ".driver");
              myUser=rbRDBMS.getString(targetName+ ".user");
              myPassword=rbRDBMS.getString(targetName+ ".password");

         }

         if (dsnMap.isEmpty()) {
             // load the JDBC-ODBC bridge driver that is included in JDK.
             try {
                  Class.forName(mySQLDriver);
                  System.out.println("\ndriver loaded.");
             } catch (Exception e) {
                  e.printStackTrace();
                  System.out.println("Can't load driver");
             }
         }

         //dsnMap
         if (!dsnMap.containsKey(targetName)) {
               // make a connection to the selected data source.
               try {
                    dsnMap.put(targetName,DriverManager.getConnection(mySQLurl,myUser,myPassword));
                    System.out.println(targetName + ": Connection opened.");
               } catch (Throwable t) {
                    System.out.println(targetName + ": Can't open connection " + t.getMessage());
               }

               // create a statement object so we can send SQL to the database.
               try {
                    Connection thisCon=(Connection) dsnMap.get(targetName);
                    dsnMap.put(targetName + "stmt",thisCon.createStatement());
                    System.out.println(targetName + ": JDBC statement created.");
               } catch (Throwable t) {
                    System.out.println(targetName + ": Can't create JDBC statement");
               }

         }
}

  private static void insertIntoTable(String targetName, String tableName,String[] strArgs, double[] dblArgs) {

        String insertString=null;
        df.applyPattern("##0.00");

        try {

          insertString = "";

          insertString="INSERT INTO "+ tableName+ " values (";

          for (int i = 0; i <strArgs.length; i++) {

               insertString = insertString  + "'" + strArgs[i] + "',";

          }

          for (int i = 0; i < dblArgs.length; i++) {
             if (i==dblArgs.length -1) {
               insertString = insertString  +  df.format(dblArgs[i])  + ")";
             } else {
               insertString = insertString  +  df.format(dblArgs[i]) + ",";
             }
          }

          Statement thisStmt=(Statement) dsnMap.get(targetName + "stmt");
          int now=thisStmt.executeUpdate(insertString);

        } catch(Throwable t) {
          System.out.println(t.getMessage());
          System.out.println(insertString) ;
        }



}

private static void insertIntoTable(String targetName, String tableName,String[] strArgs, double[] dblArgs,int[] decimalPoints,String[] replaceMiss) {

        String insertString=null;
        String numToFormat=null;

        try {

          insertString = "";

          insertString="INSERT INTO "+ tableName+ " values (";

          for (int i = 0; i <strArgs.length; i++) {

               insertString = insertString  + "'" + strArgs[i] + "',";

          }

          for (int i = 0; i < dblArgs.length; i++) {

             if (com.hyperion.essbase.calculator.Calculator.isMissing(dblArgs[i]))
                try {
                   numToFormat= new RoundDouble().getRoundAsString(Double.parseDouble(replaceMiss[i]),decimalPoints[i] ) ;
                } catch (Exception x) {
                   numToFormat=new RoundDouble().getRoundAsString(0.0,decimalPoints[i] ) ;
                }
             else
                numToFormat=new RoundDouble().getRoundAsString(dblArgs[i],decimalPoints[i]) ;

             if (i==dblArgs.length -1) {
               insertString = insertString  +  numToFormat  + ")";
             } else {
               insertString = insertString  +  numToFormat + ",";
             }
          }

          Statement thisStmt=(Statement) dsnMap.get(targetName + "stmt");
          int now=thisStmt.executeUpdate(insertString);

        } catch(Throwable t) {
          System.out.println(t.getMessage());
          System.out.println(insertString) ;
        } finally {
            numToFormat=null;
            insertString=null;

        }



}
 public static void storeRecordsToSort(String fileName, String[] strArgs, double[] dblArgs,double field, String order) {

        int sortOrder=0;
        if (order.equalsIgnoreCase("dsc") )
            sortOrder=SortRecords.DESC_ORDER;
        else
            sortOrder=SortRecords.ASC_ORDER;

        int col=1;
        try {
                  if ((int)field>0)
                      col=(int)field;

        } catch(Exception ex) {}


        SortRecords thisSort=new SortRecords(strArgs,dblArgs,col,sortOrder);

         if (!sortMap.containsKey(fileName)) {
               try {
                   sortMap.put(fileName,new ArrayList());
               } catch (Throwable t) {
                   System.out.println("Can't store record" + t.getMessage());
               }
         }
         List l = (List) sortMap.get(fileName);
         if (l!=null) {
            l.add(thisSort) ;
         }
         thisSort=null;
         l=null;

  }



  public static void closeFiles(String fileName) {

              try {

                  if (fileMap.containsKey(fileName)) {

                      FileWriter thisOut= (FileWriter) fileMap.get(fileName);
                      thisOut.close() ;
                      thisOut=null;

                      fileMap.remove(fileName);
                      System.out.println( fileName + ": file Closed");

                  }


              } catch (Exception e) {

              }
  }

  public static void sortRecords(String fileName, String rank, String delimiter,
					String textQualifier, int[] decimalPoints, String[] replaceMiss) {

              List l = null;
              SortRecords[] sorts=null;
              SortRecords[] sorted=null;

              try {

                  if (sortMap.containsKey(fileName)) {

                      l = (List) sortMap.get(fileName);
                      sorts=new SortRecords[l.size()];
                      l.toArray(sorts);

                      sorted=new SortRecords().sortThis(sorts);

                      openFile(fileName);
                      int upperLimit=sorted.length;
                      if (rank!=null && rank.trim().length() >0) {
                            try {
                                if (Integer.parseInt(rank)>0  && Integer.parseInt(rank)<=upperLimit )
                                    upperLimit=Integer.parseInt(rank);

                            } catch(Exception ex) {}
                      }

                      for (int i=0;i<upperLimit;i++ ) {
                           insertToFile(fileName, delimiter,textQualifier,sorted[i].members,sorted[i].values,decimalPoints, replaceMiss);
                      }


                      //System.runFinalization();
                      //System.gc() ;



                  }


              } catch (Exception e) {

              } finally {

                      closeFiles(fileName);

                      sorts=null;
                      sorted=null;
                      l=null;
                      sortMap.remove(fileName);

              }
         }
public static void sortRecords(String fileName, String top, String bottom, String delimiter,
						String textQualifier, int[] decimalPoints, String[] replaceMiss) {

              List l = null;
              SortRecords[] sorts=null;
              SortRecords[] sorted=null;
              int topNum=0;
              int bottomNum=0;
              boolean ascending=true;

              try {

                  if (sortMap.containsKey(fileName)) {

                      l = (List) sortMap.get(fileName);
                      sorts=new SortRecords[l.size()];
                      l.toArray(sorts);

                      sorted=new SortRecords().sortThis(sorts);

                      openFile(fileName);
                      if (top!=null && top.trim().length() >0) {
                          try {
                                topNum=Integer.parseInt(top);
                          } catch(Exception ex) {}
                      }

                      if (bottom!=null && bottom.trim().length() >0) {
                          try {
                                bottomNum=Integer.parseInt(bottom);
                          } catch(Exception ex) {}
                      }

                      if (sorted[0].sortOrder== SortRecords.DESC_ORDER) {
                          ascending=false;
                      }

                      int upperLimit=sorted.length;
                      int bottomLimit=0;
                      //int i=0;
                     // int j=0;

                      //  0,0 returns all
                      //  5,0 returns top 5
                      //  0,10 returns bottom 10

                      //  0,0 returns all
                      if (topNum ==0 && bottomNum==0) {
                            bottomLimit=upperLimit;
                      } else if  (topNum>0  && topNum<=upperLimit ) {
                            upperLimit=topNum;
                      }



                      for (int i=0;i<upperLimit;i++ ) {
                           insertToFile(fileName, delimiter,textQualifier,sorted[i].members,sorted[i].values,decimalPoints, replaceMiss );
                      }

                      for (int j=bottomLimit;j<upperLimit;j++ ) {
                           insertToFile(fileName, delimiter,textQualifier,sorted[j].members,sorted[j].values,decimalPoints, replaceMiss );
                      }


                  }


              } catch (Exception e) {

              } finally {

                      closeFiles(fileName);

                      sorts=null;
                      sorted=null;
                      l=null;
                      sortMap.remove(fileName);

              }
  }
  public static void disconnectFromDSN(String dsn) {


              try {

                  if (dsnMap.containsKey(dsn)) {

                      Statement thisStmt=(Statement) dsnMap.get(dsn + "stmt");
                      thisStmt.close() ;
                      thisStmt=null;

                      Connection thisCon=(Connection) dsnMap.get(dsn);
                      thisCon.close();
                      thisCon=null;

                      dsnMap.remove(dsn);
                      dsnMap.remove(dsn + "stmt");
                      System.out.println( dsn + ": connection Closed");

                  }


              } catch (Exception e) {

              }
  }



}